/*
 * Button Class for CYD Calculator
 *
 * Concept, Design and Implementation by: Craig A. Lindley
 * Last Update: 01/05/2025
 */

#ifndef BUTTON_H
#define BUTTON_H

// Default button dimensions
#define DEFAULT_BUTTON_WIDTH 35
#define DEFAULT_BUTTON_HEIGHT 30
#define DEFAULT_BUTTON_RADIUS 8

// Default button colors
#define DEFAULT_BUTTON_COLOR ILI9341_BLACK
#define DEFAULT_BUTTON_TEXT_COLOR ILI9341_WHITE

// Delay for changing button display state
#define BUTTON_MOMENTARY_DELAY_MS 200

class Button {

public:

  Button(int x, int y, const char *text, enum BTOKEN token) {
    _buttonX = x;
    _buttonY = y;
    _token = token;

    // Clear button text buffer
    memset(_buttonText, 0, sizeof(_buttonText));

    // Copy button text into storage
    strcpy(_buttonText, text);
  }

  enum BTOKEN pollButton(Adafruit_ILI9341 *pLcd, uint16_t touchX, uint16_t touchY) {

    // Was this button clicked?
    if ((touchX >= _buttonX) && (touchX < (_buttonX + DEFAULT_BUTTON_WIDTH)) && (touchY >= _buttonY) && (touchY < (_buttonY + DEFAULT_BUTTON_HEIGHT))) {
      // Button was clicked, process accordingly
      drawButton(pLcd, true);

      delay(BUTTON_MOMENTARY_DELAY_MS);

      drawButton(pLcd, false);

      return _token;
    } else {
      return T_NONE;
    }
  }

  void drawButton(Adafruit_ILI9341 *pLcd, boolean highLighted) {
    POINT pt;
    uint16_t fillC, textC;

    if (!highLighted) {
      fillC = DEFAULT_BUTTON_COLOR;
      textC = DEFAULT_BUTTON_TEXT_COLOR;
    } else {
      fillC = DEFAULT_BUTTON_TEXT_COLOR;
      textC = DEFAULT_BUTTON_COLOR;
    }

    // Get text dimensions
    getTextWandH(pLcd, &pt, _buttonText);

    pLcd->fillRoundRect(_buttonX, _buttonY, DEFAULT_BUTTON_WIDTH, DEFAULT_BUTTON_HEIGHT, DEFAULT_BUTTON_RADIUS, fillC);

    // Center the button's text in the button rect
    // NOTE: the +9 is used to offset the text down from the baseline
    // and is because we are using the FreeMono9pt7b 9 point font
    pLcd->setCursor(_buttonX + ((DEFAULT_BUTTON_WIDTH - pt.x) / 2),
                    _buttonY + ((DEFAULT_BUTTON_HEIGHT - pt.y) / 2) + 9);

    pLcd->setTextColor(textC);
    pLcd->print(_buttonText);
  }

protected:

  void getTextWandH(Adafruit_ILI9341 *pLcd, POINT *pt, const char *text) {

    int16_t x1, y1;
    uint16_t w, h;

    pLcd->getTextBounds(text, 0, 0, &x1, &y1, &w, &h);
    pt->x = w;
    pt->y = h;
  }

  enum BTOKEN _token;
  int _buttonX;
  int _buttonY;

  char _buttonText[10];
};

#endif
