/*
   Button Manager Class for CYD Calculator

   Concept, design and implementation by: Craig A. Lindley
   Last Update: 01/05/2025
*/

#ifndef BUTTONMANAGER_H
#define BUTTONMANAGER_H

#include <vector>

// Assuming button row height of 30 and row vertical space of 14
#define BR_0 76
#define BR_1 120
#define BR_2 164
#define BR_3 208

// Assuming button width of 35 and button horizontal space of 5
#define BC_0 1
#define BC_1 41
#define BC_2 81
#define BC_3 121
#define BC_4 161
#define BC_5 201
#define BC_6 241
#define BC_7 281

class ButtonManager {

public:

  ButtonManager() {

    // Add all Buttons to button vector
    allButtons.push_back(Button(BC_0, BR_0, "SR", T_SR));
    allButtons.push_back(Button(BC_1, BR_0, "T^X", T_10X));
    allButtons.push_back(Button(BC_2, BR_0, "Y^X", T_YX));
    allButtons.push_back(Button(BC_3, BR_0, "CHS", T_CHS));
    allButtons.push_back(Button(BC_4, BR_0, "7", T_7));
    allButtons.push_back(Button(BC_5, BR_0, "8", T_8));
    allButtons.push_back(Button(BC_6, BR_0, "9", T_9));
    allButtons.push_back(Button(BC_7, BR_0, "/", T_DIV));

    allButtons.push_back(Button(BC_0, BR_1, "SIN", T_SIN));
    allButtons.push_back(Button(BC_1, BR_1, "COS", T_COS));
    allButtons.push_back(Button(BC_2, BR_1, "TAN", T_TAN));
    allButtons.push_back(Button(BC_3, BR_1, "EEX", T_EEX));
    allButtons.push_back(Button(BC_4, BR_1, "4", T_4));
    allButtons.push_back(Button(BC_5, BR_1, "5", T_5));
    allButtons.push_back(Button(BC_6, BR_1, "6", T_6));
    allButtons.push_back(Button(BC_7, BR_1, "*", T_MULT));

    allButtons.push_back(Button(BC_0, BR_2, "D>R", T_DTOR));
    allButtons.push_back(Button(BC_1, BR_2, "R>D", T_RTOD));
    allButtons.push_back(Button(BC_2, BR_2, "FIX", T_FIX));
    allButtons.push_back(Button(BC_3, BR_2, "SWP", T_SWP));
    allButtons.push_back(Button(BC_4, BR_2, "1", T_1));
    allButtons.push_back(Button(BC_5, BR_2, "2", T_2));
    allButtons.push_back(Button(BC_6, BR_2, "3", T_3));
    allButtons.push_back(Button(BC_7, BR_2, "-", T_MINUS));

    allButtons.push_back(Button(BC_0, BR_3, "M>I", T_MMTOI));
    allButtons.push_back(Button(BC_1, BR_3, "I>M", T_ITOMM));
    allButtons.push_back(Button(BC_2, BR_3, "CLR", T_CLR));
    allButtons.push_back(Button(BC_3, BR_3, "ENT", T_ENT));
    allButtons.push_back(Button(BC_4, BR_3, "0", T_0));
    allButtons.push_back(Button(BC_5, BR_3, ".", T_DOT));
    allButtons.push_back(Button(BC_6, BR_3, "BS", T_BS));
    allButtons.push_back(Button(BC_7, BR_3, "+", T_PLUS));
  }

  // Draw the buttons on the screen
  void drawButtons(Adafruit_ILI9341 *pLcd) {

    for (auto itr : allButtons) {
      itr.drawButton(pLcd, false);
    }
  }

  enum BTOKEN pollAllButtons(Adafruit_ILI9341 *pLcd, POINT pt) {

    // Poll all buttons
    for (auto itr : allButtons) {
      BTOKEN token = itr.pollButton(pLcd, pt.x, pt.y);
      if (token != T_NONE) {
        return token;
      }
    }
    return T_NONE;
  }

protected:

  // A vector for storing all buttons
  std::vector<Button> allButtons;
};



#endif
