/*
   Display Helper Class for the CYD Calculator

   Concept, design and implementation by: Craig A. Lindley
   Last Update: 01/05/2025
*/

#ifndef DISPLAYHELPER_H
#define DISPLAYHELPER_H

#define CALC_DISPLAY_WIDTH 260
#define CALC_DISPLAY_HEIGHT 50
#define CALC_DISPLAY_RADIUS 10

#define CALC_DISPLAY_OFFSET_X 30
#define CALC_DISPLAY_OFFSET_Y 12

#define CALC_DISPLAY_COLOR ILI9341_BLUE
#define CALC_DISPLAY_OUTLINE_COLOR ILI9341_BLACK
#define CALC_DISPLAY_CONTENT_COLOR ILI9341_GREEN

class DisplayHelper {

public:

  DisplayHelper(Adafruit_ILI9341 *pLcd) {
    // Save incoming
    _pLcd = pLcd;

    // Default decimal places
    _dPlaces = 2;
  }

  // Clear and format the display area
  void clearDisplay() {
    _pLcd->fillRoundRect(CALC_DISPLAY_OFFSET_X, CALC_DISPLAY_OFFSET_Y, CALC_DISPLAY_WIDTH, CALC_DISPLAY_HEIGHT, CALC_DISPLAY_RADIUS, CALC_DISPLAY_COLOR);
    _pLcd->drawRoundRect(CALC_DISPLAY_OFFSET_X, CALC_DISPLAY_OFFSET_Y, CALC_DISPLAY_WIDTH, CALC_DISPLAY_HEIGHT, CALC_DISPLAY_RADIUS, CALC_DISPLAY_OUTLINE_COLOR);
  }

  // Set the number of decimal places to display
  void setDecimalPlaces(int dPlaces) {
    _dPlaces = dPlaces;
  }

  void displayNumber(double number) {

    // First clear the display area
    clearDisplay();

    memset(numericString, 0, sizeof(numericString));

    // Round double to specified number of decimal places
    dtostrf(number, 4, _dPlaces, numericString);

    _pLcd->setTextSize(2);
    getTextWandH(numericString);
    _pLcd->setCursor(CALC_DISPLAY_OFFSET_X + 4, CALC_DISPLAY_OFFSET_Y + ((CALC_DISPLAY_HEIGHT - pt.y) / 2) + 18);
    _pLcd->setTextColor(CALC_DISPLAY_CONTENT_COLOR);
    _pLcd->print(numericString);
    _pLcd->setTextSize(1);
  }

  // Display the input buffer as a double number is being assembled
  void displayInputBuffer(const char *inputBuffer) {
    clearDisplay();
    _pLcd->setTextSize(2);
    getTextWandH(inputBuffer);
    _pLcd->setCursor(CALC_DISPLAY_OFFSET_X + 4, CALC_DISPLAY_OFFSET_Y + ((CALC_DISPLAY_HEIGHT - pt.y) / 2) + 18);
    _pLcd->setTextColor(CALC_DISPLAY_CONTENT_COLOR);
    _pLcd->print(inputBuffer);
    _pLcd->setTextSize(1);
  }

protected:

  Adafruit_ILI9341 *_pLcd;

  char numericString[20];

  int _dPlaces;

  POINT pt;

  void getTextWandH(const char *text) {

    int16_t x1, y1;
    uint16_t w, h;

    _pLcd->getTextBounds(text, 0, 0, &x1, &y1, &w, &h);
    pt.x = w;
    pt.y = h;
  }
};








#endif
