/*
   Text Helper Class for the CYD Calculator

   Concept, design and implementation by: Craig A. Lindley
   Last Update: 01/05/2025
*/

#ifndef TEXTHELPER_H
#define TEXTHELPER_H

struct POINT {
  int x;
  int y;
};

class TextHelper {

public:

  TextHelper(Adafruit_ILI9341 *pLcd) {
    // Save incoming
    _pLcd = pLcd;
  }

  // Get the width in pixels of a text string in current font
  uint16_t getTextWidth(const char *text) {

    int16_t x1, y1;
    uint16_t w, h;

    _pLcd->getTextBounds(text, 0, 0, &x1, &y1, &w, &h);

    return w;
  }

  // Get the height in pixels of a text string in current font
  uint16_t getTextHeight(const char *text) {

    int16_t x1, y1;
    uint16_t w, h;

    _pLcd->getTextBounds(text, 0, 0, &x1, &y1, &w, &h);

    return h;
  }

  // Calculate the x offset necessary to center a text string
  int xOffsetToCenterText(const char *text) {
    return (_pLcd->width() - getTextWidth(text)) / 2;
  }

  // Draw a text string at specified location
  void drawText(int x, int y, const char *text) {
    _pLcd->setCursor(x, y);
    _pLcd->print(text);
  }

  // Draw a text string centered on display with y position
  void drawCenteredText(int y, const char *text) {
    drawText(xOffsetToCenterText(text), y, text);
  }
protected:
  
  Adafruit_ILI9341 *_pLcd;
};

#endif
